package com.frist008.nixlibrary;

import android.database.Cursor;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.Loader;
import android.support.v4.widget.CursorAdapter;
import android.support.v7.widget.RecyclerView;
import android.widget.AbsListView;
import android.widget.HeaderViewListAdapter;
import android.widget.ListAdapter;

import com.frist008.nixlibrary.recycler.CursorRecyclerAdapter;

import se.emilsjolander.stickylistheaders.StickyListHeadersListView;

@SuppressWarnings("unused")
public class LoaderUtils {

	/**
	 * If loader is null then call {@link LoaderManager#initLoader}. If loader not null and started
	 * then call {@link Loader#forceLoad} else call {@link LoaderManager#restartLoader}
	 */
	public static void forceLoader(@NonNull Fragment fragment,
	                               @Nullable LoaderManager.LoaderCallbacks callback,
	                               int loaderId) {
		forceLoader(fragment, null, callback, loaderId);
	}

	/**
	 * If loader is null then call {@link LoaderManager#initLoader}. If loader not null and started
	 * then call {@link Loader#forceLoad} else call {@link LoaderManager#restartLoader}
	 */
	public static void forceLoader(@NonNull Fragment fragment,
	                               @Nullable Bundle bundle,
	                               @Nullable LoaderManager.LoaderCallbacks callback,
	                               int loaderId) {
		if (!fragment.isDetached()) {
			forceLoader(fragment.getLoaderManager(), bundle, callback, loaderId);
		}
	}

	/**
	 * If loader is null then call {@link LoaderManager#initLoader}. If loader not null and started
	 * then call {@link Loader#forceLoad} else call {@link LoaderManager#restartLoader}
	 */
	public static void forceLoader(@NonNull LoaderManager loaderManager,
	                               @Nullable LoaderManager.LoaderCallbacks callback,
	                               int loaderId) {
		forceLoader(loaderManager, null, callback, loaderId);
	}

	/**
	 * If loader is null then call {@link LoaderManager#initLoader}. If loader not null and started
	 * then call {@link Loader#forceLoad} else call {@link LoaderManager#restartLoader}
	 */
	public static void forceLoader(@NonNull LoaderManager loaderManager,
	                               @Nullable Bundle bundle,
	                               @Nullable LoaderManager.LoaderCallbacks callback,
	                               int loaderId) {
		Loader loader = loaderManager.getLoader(loaderId);

		if (loader != null) {
			if (loader.isStarted()) {
				loader.forceLoad();
			} else {
				loaderManager.restartLoader(loaderId, bundle, callback).forceLoad();
			}
		} else {
			loaderManager.initLoader(loaderId, bundle, callback);
		}
	}

	/**
	 * If loader is null then call {@link LoaderManager#initLoader} else call {@link
	 * LoaderManager#restartLoader}
	 */
	public static void restartLoader(@NonNull Fragment fragment,
	                                 @Nullable LoaderManager.LoaderCallbacks callback,
	                                 int loaderId) {
		restartLoader(fragment, null, callback, loaderId);
	}

	/**
	 * If loader is null then call {@link LoaderManager#initLoader} else call {@link
	 * LoaderManager#restartLoader}
	 */
	public static void restartLoader(@NonNull Fragment fragment,
	                                 @Nullable Bundle bundle,
	                                 @Nullable LoaderManager.LoaderCallbacks callback,
	                                 int loaderId) {
		if (!fragment.isDetached()) {
			restartLoader(fragment.getLoaderManager(), bundle, callback, loaderId);
		}
	}

	/**
	 * If loader is null then call {@link LoaderManager#initLoader} else call {@link
	 * LoaderManager#restartLoader}
	 */
	public static void restartLoader(@Nullable LoaderManager loaderManager,
	                                 @Nullable LoaderManager.LoaderCallbacks callback,
	                                 int loaderId) {
		restartLoader(loaderManager, null, callback, loaderId);
	}

	/**
	 * If loader is null then call {@link LoaderManager#initLoader} else call {@link
	 * LoaderManager#restartLoader}
	 */
	public static void restartLoader(@Nullable LoaderManager loaderManager,
	                                 @Nullable Bundle bundle,
	                                 @Nullable LoaderManager.LoaderCallbacks callback,
	                                 int loaderId) {
		if (loaderManager != null) {
			Loader<Cursor> loader = loaderManager.getLoader(loaderId);

			if (loader != null) {
				loaderManager.restartLoader(loaderId, bundle, callback);
			} else {
				loaderManager.initLoader(loaderId, bundle, callback);
			}
		}
	}

	/**
	 * @see CursorAdapter#changeCursor(Cursor)
	 */
	public static void changeCursor(@NonNull AbsListView absListView, @Nullable Cursor cursor) {
		ListAdapter adapter = absListView.getAdapter();

		if (adapter instanceof CursorAdapter) {
			((CursorAdapter) adapter).changeCursor(cursor);
		} else {
			((CursorAdapter) ((HeaderViewListAdapter) adapter)
					.getWrappedAdapter()).changeCursor(cursor);
		}
	}

	/**
	 * @see CursorAdapter#changeCursor(Cursor)
	 */
	public static void changeCursorStickyListHeaders(@NonNull StickyListHeadersListView listView,
	                                                 @Nullable Cursor cursor) {
		ListAdapter adapter = listView.getAdapter();

		if (adapter instanceof CursorAdapter) {
			((CursorAdapter) adapter).changeCursor(cursor);
		} else {
			((CursorAdapter) ((HeaderViewListAdapter) adapter)
					.getWrappedAdapter()).changeCursor(cursor);
		}
	}

	/**
	 * @see CursorAdapter#changeCursor(Cursor)
	 */
	public static void changeCursorRecycler(@NonNull RecyclerView recyclerView,
	                                        @Nullable Cursor cursor) {
		RecyclerView.Adapter adapter = recyclerView.getAdapter();

		if (adapter instanceof CursorRecyclerAdapter) {
			((CursorRecyclerAdapter) adapter).changeCursor(cursor);
		}
	}
}