package com.frist008.nixlibrary;

import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;

import com.frist008.nixlibrary.database.Sort;

@SuppressWarnings("unused")
public class NixCursorLoader<List> implements LoaderManager.LoaderCallbacks<Cursor> {

	@NonNull
	private final FragmentActivity activity;
	@Nullable
	private final Fragment fragment;
	@Nullable
	private final Uri uri;
	@Nullable
	private final String[] projection;
	@Nullable
	private final String selection;
	@Nullable
	private final String[] selectionArgs;
	@Nullable
	private final String sortOrder;
	@Nullable
	private final OnChangeCursorBeforeListener listenerBefore;
	@Nullable
	private final OnChangeCursorAfterListener listenerAfter;
	@Nullable
	private final OnChangeWhereListener whereListener;
	private final List list;
	private Cursor lastCursor;

	@SuppressWarnings("UnusedParameters")
	public NixCursorLoader(@Nullable Fragment fragment,
	                       @NonNull FragmentActivity activity,
	                       @Nullable List list,
	                       int loaderId,
	                       @Nullable OnChangeCursorBeforeListener listenerBefore,
	                       @Nullable OnChangeCursorAfterListener listenerAfter,
	                       @Nullable OnChangeWhereListener whereListener,
	                       @Nullable Uri uri,
	                       @Nullable String[] projection,
	                       @Nullable Sort sortOrder,
	                       @Nullable String selection,
	                       @Nullable String... args) {
		this.list = list;
		this.fragment = fragment;
		this.activity = activity;
		this.listenerBefore = listenerBefore;
		this.listenerAfter = listenerAfter;
		this.whereListener = whereListener;
		this.uri = uri;
		this.projection = projection;
		this.selection = selection;
		this.selectionArgs = args;
		this.sortOrder = sortOrder != null ? sortOrder.getOrderBy() : null;

		LoaderUtils.restartLoader(this.activity.getSupportLoaderManager(), this, loaderId);
	}

	@Override
	public void onLoaderReset(Loader<Cursor> cursorLoader) {
		changeCursor(null);
	}

	@Override
	public void onLoadFinished(Loader<Cursor> loader, @Nullable Cursor cursor) {
		changeCursor(cursor);
	}

	protected void changeCursor(@Nullable Cursor cursor) {
		boolean existInstanceScreen = fragment == null || fragment.getView() != null;

		if (listenerBefore != null && existInstanceScreen) {
			listenerBefore.changeCursor(cursor);
		}

		if (list != null) {
			changeCursor(list, cursor);
		}

		if (listenerAfter != null && existInstanceScreen) {
			listenerAfter.changeCursor(cursor);
		}
	}

	protected void changeCursor(@NonNull List list, @Nullable Cursor cursor) {
		if (lastCursor != null && !lastCursor.isClosed()) {
			lastCursor.close();
		}

		lastCursor = cursor;
	}

	@Override
	public Loader<Cursor> onCreateLoader(int id, Bundle bundle) {
		String newSelection = null;
		String[] newArgs = null;

		if (whereListener != null) {
			NixWhereLoaderContainer
					container = whereListener.getWhere(new NixWhereLoaderContainer());
			newSelection = container.getSelect();
			newArgs = container.getArgs();
		}

		return new CursorLoader(activity,
		                        uri,
		                        projection,
		                        newSelection == null ? selection : newSelection,
		                        newArgs == null ? selectionArgs : newArgs,
		                        sortOrder);
	}
}
